; ********************************************************************************
; zcode PWM calculation and update functions
;
;-------------------------------------------
; Fade Direction
; 
_colActivate  	btfss  	flags, rxNewCol	; is colour data stale?
        	return  		; exit if it is
      	bcf     	flags, rxNewCol	; else clear flag

	movfw   	fadeLatch	; get new fade rate
	bz	_noFade	; do noFade code if fade rate == 0

	movwf   	fadeRate	; else save fade rate
	movwf	fadeCounter	; 
	movfw   	redLatch 	; transfer colour data from
        	movwf   	redNEW 	; receive latch to new data
        	movfw   	grnLatch 
        	movwf   	grnNEW 
        	movfw   	bluLatch 
        	movwf   	bluNEW	
			; and then compute fade rate

;-------------------------------------------
; Calculate the difference between current and new
; PWM value for each colour. Set fade direction to +/-1
	
	
_calcFade	movlw	(1<<redFade | 1<<grnFade | 1<<bluFade)
	movwf	fadeflag	; preset fade flag
	movlw	0x01	; preset fade direction to +1 for each colour
	movwf	rdir
	movwf	gdir
	movwf	bdir
	
	movfw	red
	subwf	redNEW,W	; subtract current from new (W = current - new)
	bc	_rabs	; branch if C=1  (current > new)
	decf	rdir,F	; rdir is 1, decrement twice
	decf	rdir,F	; to make rdir -1.
	sublw	0x00	; sub W from 0 to make W positive
_rabs	movwf	Rdif	; save difference between current and new

	movfw	grn
	subwf	grnNEW,W	; subtract current from new (W = current - new)
	bc	_gabs	; branch if C=1  (current > new)
	decf	gdir,F	; rdir is 1, decrement twice
	decf	gdir,F	; to make rdir -1.
	sublw	0x00	; sub W from 0 to make W positive
_gabs	movwf	Gdif	; save difference between current and new

	movfw	blu
	subwf	bluNEW,W	; subtract current from new (W = current - new)
	bc	_babs	; branch if C=1  (current > new)
	decf	bdir,F	; rdir is 1, decrement twice
	decf	bdir,F	; to make rdir -1.
	sublw	0x00	; sub W from 0 to make W positive
_babs	movwf	Bdif	; save difference between current and new


;-------------------------------------------
; Find largest difference between current and new RGB values
; 	
	
_calcLargest	movfw	Rdif
	subwf	Gdif,W
	bc	_chkGB
	
	movfw	Rdif
	subwf	Bdif,W
	movfw	Bdif
	bc	_done
	movfw	Rdif
	goto	_done
	
_chkGB	movfw	Gdif
	subwf	Bdif,W
	movfw	Bdif
	skpc	
	movfw	Gdif

_done	sublw	0x00
	movwf	CntBase
	movwf	Rcnt
	movwf	Gcnt
	movwf	Bcnt
	
	return

;-----------------------------------------------------
; Test if current fade completed
	
_fadeState	movf	fadeflag,F	; test if fadeflag is zero
	skpnz
	return
	
;------------------------------------------------------
; fade	
	
_fading	decfsz	fadeCounter,F	; decrement fade counter
	return		; return if not zero
	movfw	fadeRate	; else reload fade counter
	movwf	fadeCounter

;----------------------------------------
; Fade stepper
;

_Rfade	movfw	redNEW
	xorwf	red,W
	bnz	$+3
	bcf	fadeflag,redFade
	goto	_Gfade

	movfw	Rdif
	addwf	Rcnt,F
	skpc	
	goto	_Gfade
	movfw	CntBase
	addwf	Rcnt,F
	movfw	rdir
	addwf	red,F
	
_Gfade	movfw	grnNEW
	xorwf	grn,W
	bnz	$+3
	bcf	fadeflag,grnFade
	goto	_Bfade

	movfw	Gdif
	addwf	Gcnt,F
	skpc	
	goto	_Bfade
	movfw	CntBase
	addwf	Gcnt,F
	movfw	gdir
	addwf	grn,F

_Bfade 	movfw	bluNEW
	xorwf	blu,W
	bnz	$+3
	bcf	fadeflag,bluFade
	return
	
	movfw	Bdif
	addwf	Bcnt,F
	skpc	
	return	
	movfw	CntBase
	addwf	Bcnt,F
	movfw	bdir
	addwf	blu,F
	return

;----------------------------------------
; No Fade colour transfer
;
; if faderate == 0 transfer colour data directly without any fading.

_noFade	movfw	redLatch	; No fade so load
	movwf	red	; pwm variables
	movfw	grnLatch	; with new data
	movwf	grn	; to get instant
	movfw	bluLatch	; colour change
	movwf	blu	; ...
	clrf	fadeflag	; clear fade flags so code doesn't attempt to fade
	return		; return